import aiohttp
from typing import Any, Dict
from .tessie_wrapper import tessieRequest


async def set_scheduled_charging(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    timeMins: int,
    enable: bool = True,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_scheduled_charging",
        api_key,
        params={
            "time": timeMins,
            "enable": str(enable).lower(),
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def set_scheduled_departure(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    departure_time_mins: int,
    end_off_peak_time_mins: int,
    enable: bool = False,
    preconditioning_enabled: bool = False,
    preconditioning_weekdays_only: bool = False,
    off_peak_charging_enabled: bool = False,
    off_peak_charging_weekdays_only: bool = False,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_scheduled_departure",
        api_key,
        params={
            "enable": str(enable).lower(),
            "departure_time": departure_time_mins,
            "preconditioning_enabled": str(preconditioning_enabled).lower(),
            "preconditioning_weekdays_only": str(preconditioning_weekdays_only).lower(),
            "off_peak_charging_enabled": str(off_peak_charging_enabled).lower(),
            "off_peak_charging_weekdays_only": str(
                off_peak_charging_weekdays_only
            ).lower(),
            "end_off_peak_time": end_off_peak_time_mins,
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )



import aiohttp
from typing import Any, Dict, List

from tessie_api.literals import DistanceFormat, TemperatureFormat
from .tessie_wrapper import tessieRequest


async def get_drives(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    fromUnix: int,
    toUnix: int,
    origin_latitude: int,
    origin_lonitude: int,
    origin_radius: int,
    destination_latitude: int,
    destination_longitude: int,
    destination_radius: int,
    tag: str,
    driver_profile: str,
    minimum_distance: int,
    distance_format: DistanceFormat = "km",
    temperature_format: TemperatureFormat = "c",
    timezone: str = "UTC",
    exclude_origin: bool = False,
    exclude_destination: bool = False,
    exclude_tag: bool = False,
    exclude_driver_profile: bool = False,
    format: str = "Format",
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/drives",
        api_key,
        params={
            "distance_format": distance_format,
            "temperature_format": temperature_format,
            "fromUnix": fromUnix,
            "toUnix": toUnix,
            "timezone": timezone,
            "origin_latitude": origin_latitude,
            "origin_lonitude": origin_lonitude,
            "origin_radius": origin_radius,
            "exclude_origin": str(exclude_origin).lower(),
            "destination_latitude": destination_latitude,
            "destination_longitude": destination_longitude,
            "destination_radius": destination_radius,
            "exclude_destination": str(exclude_destination).lower(),
            "tag": tag,
            "exclude_tag": str(exclude_tag).lower(),
            "driver_profile": driver_profile,
            "exclude_driver_profile": str(exclude_driver_profile).lower(),
            "format": format,
            "minimum_distance": minimum_distance,
        },
    )


async def get_driving_path(
    session: aiohttp.ClientSession, vin: str, api_key: str, fromUnix: int, toUnix: int
) -> Dict[str, Any]:
    return await tessieRequest(
        session, "GET", f"/{vin}/path", api_key, params={"from": fromUnix, "to": toUnix}
    )


async def set_tag(
    session: aiohttp.ClientSession, vin: str, api_key: str, drives: List[int], tag: str
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "POST",
        f"/{vin}/drives/set_tag",
        api_key,
        params={"drives": drives, "tag": tag},
    )



import aiohttp
from typing import Any, Dict, Optional


async def tessieRequest(
    session: aiohttp.ClientSession,
    method: str,
    path: str,
    api_key: str,
    params: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    url = f"https://api.tessie.com{path}"
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json",
    }
    async with session.request(method, url, headers=headers, params=params) as response:
        response.raise_for_status()  # Will raise aiohttp.ClientResponseError on 4xx or 5xx
        return await response.json()



import aiohttp
from typing import Any, Dict

import pytz

from tessie_api.literals import DistanceFormat, Format, TemperatureFormat
from .tessie_wrapper import tessieRequest


async def get_historical_states(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    fromUnix: int,
    toUnix: int,
    interval: int,
    condensed: bool = False,
    timezone: str = "UTC",
    distance_format: DistanceFormat = "km",
    temperature_format: TemperatureFormat = "c",
    format: Format = "json",
) -> Dict[str, Any]:
    if timezone not in pytz.all_timezones:
        raise ValueError(
            "Invalid timezone. Please sure the passed value is a valid IANA timezone."
        )

    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/states",
        api_key,
        params={
            "from": fromUnix,
            "to": toUnix,
            "interval": interval,
            "condense": str(condensed).lower(),
            "timezone": timezone,
            "distance_format": distance_format,
            "temperature_format": temperature_format,
            "format": format,
        },
    )


async def get_last_idle_state(
    session: aiohttp.ClientSession, vin: str, api_key: str
) -> Dict[str, Any]:
    return await tessieRequest(session, "GET", f"/{vin}/last_idle_state", api_key)


async def get_consumption_since_charge(
    session: aiohttp.ClientSession, vin: str, api_key: str
) -> Dict[str, Any]:
    return await tessieRequest(
        session, "GET", f"/{vin}/consumption_since_charge", api_key
    )



import aiohttp
from typing import Any, Dict

from tessie_api.literals import ClimateKeeperMode, Seat
from .tessie_wrapper import tessieRequest


async def start_climate_preconditioning(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/start_climate",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def stop_climate(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/stop_climate",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def set_temperature(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    temperature: int,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_temperatures",
        api_key,
        params={
            "temperature": temperature,
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def set_seat_heat(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    seat: Seat,
    level: int = 3,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_seat_heat",
        api_key,
        params={
            "seat": seat,
            "level": level,
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def start_defrost(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/start_max_defrost",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def stop_defrost(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/stop_max_defrost",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def start_steering_wheel_heater(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/start_steering_wheel_heater",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def stop_steering_wheel_heater(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/stop_steering_wheel_heater",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def set_bioweapon_defense_mode(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_bioweapon_mode",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )


async def set_climate_keeper_mode(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    mode: ClimateKeeperMode,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/set_climate_keeper_mode",
        api_key,
        params={
            "mode": mode,
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )



import aiohttp
from typing import Any, Dict

from .literals import DistanceFormat
from .tessie_wrapper import tessieRequest


async def get_battery_health(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    from_time: int = None,
    to_time: int = None,
    distance_format: DistanceFormat = "km",
) -> Dict[str, Any]:
    params = {
        k: v
        for k, v in {
            "from": from_time,
            "to": to_time,
            "distance_format": distance_format,
        }.items()
        if v is not None
    }
    return await tessieRequest(
        session, "GET", f"/{vin}/battery_health", api_key, params=params
    )



import os
from dotenv import load_dotenv
import aiohttp
import pytest

from tessie_api import get_state_of_all_vehicles

@pytest.mark.asyncio
async def test_can_get_vehicles():
    load_dotenv()  # take environment variables from .env.
    async with aiohttp.ClientSession() as session:  # ClientSession is created here and will be closed when exiting the block
        data = await get_state_of_all_vehicles(session=session, api_key=os.getenv("TESSIE_KEY"), only_active=True)
        vin_to_check = os.getenv("TESLA_VIN")
        assert any(result["vin"] == vin_to_check for result in data["results"])



import aiohttp
from typing import Any, Dict
from .tessie_wrapper import tessieRequest


async def trigger_homelink(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    retry_duration: int = 40,
    wait_for_completion: bool = True,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/command/trigger_homelink",
        api_key,
        params={
            "retry_duration": retry_duration,
            "wait_for_completion": str(wait_for_completion).lower(),
        },
    )



import aiohttp
from typing import Any, Dict

from tessie_api.literals import MapStyle
from .tessie_wrapper import tessieRequest


async def get_state(
    session: aiohttp.ClientSession, vin: str, api_key: str, use_cache: bool = True
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/state",
        api_key,
        params={"use_cache": str(use_cache).lower()},
    )


async def get_state_of_all_vehicles(
    session: aiohttp.ClientSession, api_key: str, only_active: bool = True
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/vehicles",
        api_key,
        params={"only_active": str(only_active).lower()},
    )


async def get_location(
    session: aiohttp.ClientSession, vin: str, api_key: str
) -> Dict[str, Any]:
    return await tessieRequest(session, "GET", f"/{vin}/location", api_key)


async def get_weather(
    session: aiohttp.ClientSession, vin: str, api_key: str
) -> Dict[str, Any]:
    return await tessieRequest(session, "GET", f"/{vin}/weather", api_key)


async def get_map(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    width: int = 300,
    height: int = 300,
    zoom: int = 13,
    marker_size: int = 75,
    style: MapStyle = "light",
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/map",
        api_key,
        params={
            "width": width,
            "heigth": height,
            "zoom": zoom,
            "marker_size": marker_size,
            "style": style,
        },
    )



import aiohttp
from typing import Any, Dict

from tessie_api.literals import DistanceFormat, Format
from .tessie_wrapper import tessieRequest


async def get_charges(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    origin_latitude: int,
    origin_longitude: int,
    origin_radius: int,
    fromUnix: int,
    toUnix: int,
    minimum_energy_added: int,
    distance_format: DistanceFormat = "km",
    format: Format = "json",
    superchargers_only: bool = False,
    exclude_origin: bool = False,
    timezone: str = "UTC",
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/charges",
        api_key,
        params={
            "distance_format": distance_format,
            "format": format,
            "superchargers_only": str(superchargers_only).lower(),
            "origin_latitude": origin_latitude,
            "origin_longitude": origin_longitude,
            "origin_radius": origin_radius,
            "exclude_origin": str(exclude_origin).lower(),
            "timezone": timezone,
            "from": fromUnix,
            "to": toUnix,
            "minimum_energy_added": minimum_energy_added,
        },
    )


async def set_charge_cost(
    session: aiohttp.ClientSession,
    vin: str,
    api_key: str,
    charge_id: int,
    cost: int,
) -> Dict[str, Any]:
    return await tessieRequest(
        session,
        "GET",
        f"/{vin}/charges/{charge_id}/set_cost",
        api_key,
        params={"cost": cost},
    )



from typing import Literal
from enum import Enum


MapStyle = Literal["light", "dark"]
DistanceFormat = Literal["mi", "km"]
TemperatureFormat = Literal["c", "f"]
Format = Literal["json", "csv"]
Seat = Literal[
    "all",
    "front_left",
    "front_right",
    "rear_left",
    "rear_center",
    "rear_right",
    "third_row_left",
    "third_row_right",
]


class ClimateKeeperMode(Enum):
    DISABLE = 0
    KEEP_MODE = 1
    DOG_MODE = 2
    CAMP_MODE = 3



import aiohttp
from typing import Any, Dict
from .tessie_wrapper import tessieRequest


async def get_tire_pressure(
    session: aiohttp.ClientSession, vin: str, api_key: str
) -> Dict[str, Any]:
    return await tessieRequest(session, "GET", f"/{vin}/tire_pressure", api_key)



from .battery_health import get_battery_health
from .boombox import boombox
from .charges import get_charges
from .charges import set_charge_cost
from .charging import start_charging
from .charging import stop_charging
from .charging import set_charge_limit
from .charging import set_charging_amps
from .charging import open_unlock_charge_port
from .charging import close_charge_port
from .climate import start_climate_preconditioning
from .climate import stop_climate
from .climate import set_temperature
from .climate import set_seat_heat
from .climate import start_defrost
from .climate import stop_defrost
from .climate import start_steering_wheel_heater
from .climate import stop_steering_wheel_heater
from .climate import set_bioweapon_defense_mode
from .climate import set_climate_keeper_mode
from .current_state import get_state
from .current_state import get_state_of_all_vehicles
from .current_state import get_location
from .current_state import get_weather
from .current_state import get_map
from .doors import lock
from .doors import unlock
from .drives import get_drives
from .drives import get_driving_path
from .drives import set_tag
from .historical_states import get_historical_states
from .historical_states import get_last_idle_state
from .historical_states import get_consumption_since_charge
from .homelink import trigger_homelink
from .horn import honk
from .idles import get_idles
from .keyless_driving import enable_keyless_driving
from .lights import flash_lights
from .scheduling import set_scheduled_charging
from .scheduling import set_scheduled_departure
from .sentry_mode import enable_sentry_mode
from .sentry_mode import disable_sentry_mode
from .share import share
from .software import schedule_software_update
from .software import cancel_software_update
from .speed_limit import set_speed_limit
from .speed_limit import enable_speed_limit
from .speed_limit import disable_speed_limit
from .speed_limit import clear_speed_limit_pin
from .status import get_status
from .sunroof import vent_sunroof
from .sunroof import close_sunroof
from .tire_pressure import get_tire_pressure
from .trunks import open_front_trunk
from .trunks import open_close_rear_trunk
from .valet_mode import enable_valet_mode
from .valet_mode import disable_valet_mode
from .wake import wake
from .windows import vent_windows
from .windows import close_windows
from .literals import (
    MapStyle,
    DistanceFormat,
    TemperatureFormat,
    Format,
    Seat,
    ClimateKeeperMode,
)
